package com.hzya.frame.plugin.lets.plugin.sales;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.beanutil.BeanUtil;
import com.hzya.frame.plugin.lets.entity.*;
import com.hzya.frame.plugin.lets.ofs.dao.ITocofsSaleoutDao;
import com.hzya.frame.plugin.lets.ofs.dao.ITocofsSaleoutDetailedDao;
import com.hzya.frame.plugin.lets.ofs.entity.TocofsSaleoutDetailedEntity;
import com.hzya.frame.plugin.lets.ofs.entity.TocofsSaleoutEntity;
import com.hzya.frame.plugin.lets.ofs.service.ITocofsSaleoutService;
import com.hzya.frame.plugin.lets.ofsvo.QueryOfsSoSaleOutVo;
import com.hzya.frame.plugin.lets.ofsvo.QueryOfsSoSaleVo;
import com.hzya.frame.plugin.lets.resultvo.CalculateDateVo;
import com.hzya.frame.plugin.lets.util.BasicArchivesCacheUtil;
import com.hzya.frame.plugin.lets.util.DateStrUtil;
import com.hzya.frame.split.SplitListByCountUtil;
import com.hzya.frame.ttxofs.dto.InterfaceParamDto;
import com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.SaleOutReturnMessageDto;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.DetailsDto;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDetailsDto;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDto;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.SaleReturnMessageDto;
import com.hzya.frame.ttxofs.service.OfsUnifiedService;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;
import java.util.concurrent.locks.ReentrantLock;

/**
 * OFS销售出库单(TOC)生成U8C销售订单
 *
 * @author makejava
 * @since 2023-08-16 15:49:55
 */
public class SoSaleOutPluginInitializerToC extends PluginBaseEntity {

    Logger logger = LoggerFactory.getLogger(SoSaleOutPluginInitializerToC.class);

    private static final ReentrantLock LOCK = new ReentrantLock(true);

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "com.hzya.frame.plugin.sowow.plugin.sales.SoSaleOutPluginInitializerToC";
    }

    @Override
    public String getPluginName() {
        return "OFS销售出库单(TOC)生成U8C销售订单";
    }

    @Override
    public String getPluginLabel() {
        return "OFS销售出库单(TOC)生成U8C销售订单";
    }

    @Override
    public String getPluginType() {
        return "3";
    }

    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        return null;
    }

    @Autowired
    private OfsUnifiedService ofsUnifiedService;

    @Autowired
    private BasicArchivesCacheUtil basicArchivesCacheUtil;

    @Autowired
    private ITocofsSaleoutService iTocofsSaleoutService;

    @Autowired
    private ITocofsSaleoutDao iTocofsSaleoutDao;

    @Autowired
    private ITocofsSaleoutDetailedDao iTocofsSaleoutDetailedDao;

    /**
     * 按指定时间拉取
     *
     * @author liuyang
     */
    public void startImplement(String dateStr, String summaryOrderNumber) {
        String threadNameStr = StrUtil.format("OFS销售出库(TOC)同步U8C销售订单 同步日期：{} 汇总维度：{}", dateStr, summaryOrderNumber);
        logger.info(threadNameStr);

        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                LOCK.lock();
                try {
                    // TODO 注意过滤TOC店铺
                    CalculateDateVo calculateDateVo = DateStrUtil.calculateCalculateEntireDayPeriod(dateStr);

                    List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDetailsDto> headerDetailsDtoList = new ArrayList<>();
                    QueryOfsSoSaleOutVo queryOfsSoSaleOutVo = new QueryOfsSoSaleOutVo();
                    queryOfsSoSaleOutVo.setCreated_start("2024-03-27 12:03:30");
                    queryOfsSoSaleOutVo.setCreated_end("2024-03-27 12:04:00");
                    queryOfsSoSaleOutVo.setClientCode("LETS");
                    //        queryOfsSoSaleOutVo.setInternalInstructionType("SALES");
                    queryOfsSoSaleOutVo.setCompanyCode("LETS");
                    //        queryOfsSoSaleOutVo.setStatus(900L);
                    queryOfsSoSaleOutVo.setPageNo(1L);
                    queryOfsSoSaleOutVo.setPageSize(50L);
                    queryOfsSaleOrder(queryOfsSoSaleOutVo, headerDetailsDtoList, 1L);
                    logger.info("数据返回行数：{}", headerDetailsDtoList.size());
                    if (headerDetailsDtoList.size() > 0) {
                        //保存到mysql
                        batchInsert(headerDetailsDtoList);
                        //过滤成功的数据
                        //执行推送主逻辑
                    } else {
                        logger.info("没有查询到任何数据！不需要同步");
                    }
                } catch (Exception e) {
                    logger.error("startImplement方法抛出异常", e);
                } finally {
                    LOCK.unlock();
                }
            }
        }, threadNameStr);
        thread.start();
        try {
            thread.join();
        } catch (Exception e) {
            logger.error("thread.join()异常", e);
        }
    }

    /**
     * 保存抓取到的数据到mysql底表
     *
     * @param headerDetailsDtoList 数据行
     */
    private void batchInsert(List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDetailsDto> headerDetailsDtoList) throws Exception {
        if (headerDetailsDtoList != null && headerDetailsDtoList.size() > 0) {
            List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDto> headerDetailsDtoList1 = new ArrayList<>();
            List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.DetailsDto> headerDetailsDtoList2 = new ArrayList<>();

            for (com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDetailsDto index : headerDetailsDtoList) {
                com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDto header = index.getHeader();// 主表
                List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.DetailsDto> details = index.getDetails();//明细表

                headerDetailsDtoList1.add(header);
                headerDetailsDtoList2.addAll(details);
            }

            //每250作为一个批次插入主表，根据主键（id）判断是否重复，如果重复的，则不进行插入
            List<List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDto>> ofssaleorderoutsearchList = SplitListByCountUtil.splitListByCount(headerDetailsDtoList1, 250);
            for (int i = 0; i < ofssaleorderoutsearchList.size(); i++) {
                List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDto> headerDtoList = ofssaleorderoutsearchList.get(i);
                List<TocofsSaleoutEntity> tocofsSaleoutEntityList = copyHeaderDto(headerDtoList);
                if (tocofsSaleoutEntityList.size() > 0) {
                    iTocofsSaleoutDao.entityInsertBatchV2(tocofsSaleoutEntityList);
                }
            }

            //插入明细表
            List<List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.DetailsDto>> detailsDtoList = SplitListByCountUtil.splitListByCount(headerDetailsDtoList2, 250);
            for (int i = 0; i < detailsDtoList.size(); i++) {
                List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.DetailsDto> detailsDtos = detailsDtoList.get(i);
                List<TocofsSaleoutDetailedEntity> tocofsSaleoutDetailedEntities = copyDetailsDto(detailsDtos);
                if (tocofsSaleoutDetailedEntities.size() > 0) {
                    iTocofsSaleoutDetailedDao.entityInsertBatchV2(tocofsSaleoutDetailedEntities);
                }
            }
        }
    }

    /**
     * 查询OFS结果集
     *
     * @param queryOfsSoSaleOutVo  查询参数
     * @param headerDetailsDtoList 查询结果集
     * @param pageNo               从第pageNo页开始查询
     */
    private void queryOfsSaleOrder(QueryOfsSoSaleOutVo queryOfsSoSaleOutVo, List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDetailsDto> headerDetailsDtoList, Long pageNo) throws Exception {

        Long pageSize = 200L;
        queryOfsSoSaleOutVo.setPageNo(pageNo);
        queryOfsSoSaleOutVo.setPageSize(pageSize);

        InterfaceParamDto interfaceParamDto = new InterfaceParamDto();
        interfaceParamDto.setApi("ofs.shipment.search");
        interfaceParamDto.setData(JSON.toJSONString(queryOfsSoSaleOutVo));
        SaleOutReturnMessageDto saleOutReturnMessageDto = (SaleOutReturnMessageDto) ofsUnifiedService.unified(interfaceParamDto);
        if (saleOutReturnMessageDto != null) {
            if ("false".equals(saleOutReturnMessageDto.getError()) && "0".equals(saleOutReturnMessageDto.getCode()) && "Success".equals(saleOutReturnMessageDto.getMsg())) {
                List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDetailsDto> headerDetailsDtoList1 = saleOutReturnMessageDto.getData();
                if (headerDetailsDtoList1 != null && headerDetailsDtoList1.size() > 0) {
                    headerDetailsDtoList.addAll(headerDetailsDtoList1);

                    //TODO OFS没有提供分页字段
                }
            } else {
                //2024年7月30日 13:47:04 查询失败
                logger.error("查询失败，失败原因:{}", JSON.toJSON(saleOutReturnMessageDto));
            }
        } else {
            logger.error("returnMessageBasics为空！queryOfsSoSaleOutVo对象结果：{}", JSON.toJSON(queryOfsSoSaleOutVo));
        }
    }

    public void queryOfsSaleOrder() throws Exception {
        QueryOfsSoSaleOutVo queryOfsSoSaleOutVo = new QueryOfsSoSaleOutVo();
        queryOfsSoSaleOutVo.setCreated_start("2024-03-27 12:03:30");
        queryOfsSoSaleOutVo.setCreated_end("2024-03-27 12:04:00");
        queryOfsSoSaleOutVo.setClientCode("LETS");
//        queryOfsSoSaleOutVo.setInternalInstructionType("SALES");
        queryOfsSoSaleOutVo.setCompanyCode("LETS");
//        queryOfsSoSaleOutVo.setStatus(900L);
        queryOfsSoSaleOutVo.setPageNo(1L);
        queryOfsSoSaleOutVo.setPageSize(50L);

        InterfaceParamDto interfaceParamDto = new InterfaceParamDto();
        interfaceParamDto.setApi("ofs.shipment.search");
        interfaceParamDto.setData(JSON.toJSONString(queryOfsSoSaleOutVo));
        SaleOutReturnMessageDto saleOutReturnMessageDto = (SaleOutReturnMessageDto) ofsUnifiedService.unified(interfaceParamDto);
        System.out.println(saleOutReturnMessageDto);
    }

    /**
     * 代码同步逻辑
     */
    private void implement(List<HeaderDetailsDto> headerDetailsDtoList) {
        if (headerDetailsDtoList.size() > 0) {
            for (HeaderDetailsDto headerDetailsDto : headerDetailsDtoList) {
                HeaderDto header = headerDetailsDto.getHeader();
                List<DetailsDto> details = headerDetailsDto.getDetails();

                //生成业务日期
                String generateBusinessDate = createGenerateBusinessDate(header);

                try {

                } catch (Exception e) {
                    logger.error("数据转换过程中抛出异常：{}", e);
                }
            }
        }
    }

    /**
     * 2024年7月31日 10:34:09
     * 生成业务日期，以发货时间作为业务日期
     *
     * @author liuyang
     */
    private String createGenerateBusinessDate(HeaderDto headerDto) {
        if (headerDto != null && headerDto.getShipAt() != null) {
            //TOC以出库日期作为业务日期
            String shipAt = headerDto.getShipAt();
            String businessFormat = null;
            try {
                Date dbill = DateUtil.parse(shipAt);
                businessFormat = DateUtil.format(dbill, "yyyy-MM-dd");
            } catch (Exception e) {
                e.printStackTrace();
                logger.error("业务日期转换失败", e);
            }
            return businessFormat;
        } else {
            logger.error("生成出库日期失败，或者shipAt为空! json：{}", JSON.toJSON(headerDto));
            Assert.state(false, "生成出库日期失败，或者shipAt为空! json：{}", JSON.toJSON(headerDto));
            return null;
        }
    }

    /**
     * 档案转换
     *
     * @author liuyang
     */
    private void queryBasicArchives(List<HeaderDetailsDto> headerDetailsDtoList) {
        if (headerDetailsDtoList.size() > 0) {
            try {
                //初始化档案缓存
                basicArchivesCacheUtil.initCache();

                for (int i = 0; i < headerDetailsDtoList.size(); i++) {
                    HeaderDetailsDto headerDetailsDto = headerDetailsDtoList.get(i);

                    //表头公司：通过名称匹配U8C公司
                    BdCorpEntity bdCorpEntity = BasicArchivesCacheUtil.stringBdCorpEntityMap.get("上海娜美诺丝化妆品有限公司");

                    //发货库存组织
                    BdCalbodyEntity bdCalbodyEntity = BasicArchivesCacheUtil.stringBdCalbodyEntityMap.get(bdCorpEntity.getPkCorp());
                    Assert.notNull(bdCalbodyEntity, "根据公司主键没有查询到发货库存组织 公司主键：{}", bdCorpEntity.getPkCorp());

                    //发货仓库
                    //发货公司

                    //收货库存组织
                    //收货仓库

                    //销售组织
                    //业务部门
                    //客商档案
                }
            } catch (Exception e) {
                logger.error("OFS销售出库单关联查询U8C档案失败", e);
            } finally {
                try {
                    basicArchivesCacheUtil.clearCache();
                } catch (Exception e) {
                    logger.info("清理档案缓存失败", e);
                }
            }
        } else {
            logger.info("queryBasicArchives方法headerDetailsDtoList.size为0");
        }
    }

    /**
     * bean copy
     * 2024年7月31日 10:34:09
     */
    private List<TocofsSaleoutEntity> copyHeaderDto(List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDto> headerDtoList) {
        List<TocofsSaleoutEntity> tocofsSaleoutEntityList = new ArrayList<>();
        if (headerDtoList != null && headerDtoList.size() > 0) {
            for (int i = 0; i < headerDtoList.size(); i++) {
                com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.HeaderDto headerDto = headerDtoList.get(i);
                TocofsSaleoutEntity tocofsSaleoutEntity = new TocofsSaleoutEntity();
//                BeanUtils.copyProperties(headerDto, tocofsSaleoutEntity);
                BeanUtil.copyPropertiesV2(headerDto, tocofsSaleoutEntity);
                tocofsSaleoutEntityList.add(tocofsSaleoutEntity);
            }
        }
        return tocofsSaleoutEntityList;
    }

    /**
     * bean copy
     * 2024年8月2日 16:04:11
     */
    private List<TocofsSaleoutDetailedEntity> copyDetailsDto(List<com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.DetailsDto> detailsDtos) {
        List<TocofsSaleoutDetailedEntity> tocofsSaleoutDetailedEntityArrayList = new ArrayList<>();
        if (detailsDtos != null && detailsDtos.size() > 0) {
            for (int i = 0; i < detailsDtos.size(); i++) {
                com.hzya.frame.ttxofs.dto.ofssaleorderoutsearch.DetailsDto detailsDto = detailsDtos.get(i);
                TocofsSaleoutDetailedEntity tocofsSaleoutDetailedEntity = new TocofsSaleoutDetailedEntity();
//                BeanUtils.copyProperties(detailsDto, tocofsSaleoutDetailedEntity);
                BeanUtil.copyPropertiesV2(detailsDto, tocofsSaleoutDetailedEntity);
                tocofsSaleoutDetailedEntityArrayList.add(tocofsSaleoutDetailedEntity);
            }
        }
        return tocofsSaleoutDetailedEntityArrayList;
    }
}